//Geoget2 category library


//..............................................................................
//Category v 0.1.1

//..............................................................................
//Authors:
//Geby
//Medwyn


//..............................................................................
//Changelog:
// v0.1 - Initial release
// v0.1.1 - changed from unit to includable source


type
  TCatCallback = procedure(const geo: tgeo; const wpt: TWpt; category: string);
  
var
  cat_db: TSqliteDatabase;
  cat_callback: TCatCallback;
  
//Starts new transaction
procedure CatBeginUpdate;
begin
  cat_db.Start('','');
end;

//Commits the transaction
procedure CatEndUpdate;
begin
  cat_db.Commit('');
end;

//Adds geocache to category database
procedure CatAddGC(GC: TGeo; category: string);
begin
  cat_db.AddParamInt(':pkey', GC.Key);
  cat_db.AddParamInt(':ptab', 0);
  cat_db.AddParamText(':cat', category);
  cat_db.ExecSQL('INSERT INTO cat (ptrkey, ptrtab, category) VALUES (:pkey, :ptab, :cat)');
end;

//Adds additional waypoint to category database
procedure CatAddWpt(Wpt: TWpt; category: string);
begin
  cat_db.AddParamInt(':pkey', Wpt.Key);
  cat_db.AddParamInt(':ptab', 1);
  cat_db.AddParamText(':cat', category);
  cat_db.ExecSQL('INSERT INTO cat (ptrkey, ptrtab, category) VALUES (:pkey, :ptab, :cat)');
end;

//Sorts the database and for each point calls the Cat_Callback method.
procedure CatSort;
var
  tab: TSqliteTable;
  geo: TGeo;
  wpt: TWpt;
  x: integer;
  category: string;
  key: int64;
begin
  if cat_callback = nil then
    exit;
  geo := TGeo.Create;
  wpt := TWpt.Create;
  try
    tab := cat_db.GetTable('SELECT * FROM cat ORDER BY category', false);
    try
      while not tab.EOF do
      begin
        key := tab.FieldAsInteger(tab.FieldIndex['ptrkey']);
        x := tab.FieldAsInteger(tab.FieldIndex['ptrtab']);
        category := tab.FieldAsString(tab.FieldIndex['category']);
        if x = 0 then
        begin
          geo.LoadByKey(key);
          Cat_Callback(geo, nil, category);
        end
        else
        begin
          wpt.LoadByKey(key);
          Cat_Callback(nil, wpt, category);
        end;
        tab.next;
      end;
    finally
      tab.free;
    end;
  finally
    wpt.free;
    geo.free;
  end;
end;

procedure catInit;
begin
  cat_callback := nil;
  cat_db := TSqliteDatabase.Create('');
  cat_db.ExecSQL('PRAGMA page_size = 4096');
  cat_db.ExecSQL(
    'CREATE TABLE cat (key INTEGER PRIMARY KEY, '
    + 'ptrkey INTEGER,'
    + 'ptrtab INTEGER,'
    + 'category TEXT)');
  cat_db.ExecSQL('CREATE INDEX catidx ON cat (category)');
end;

//Returns categories in category database. 
procedure CatCategories(const value: TStrings);
begin
  cat_db.GetTableStrings('SELECT distinct(category) FROM cat ORDER BY category', value);
end;

//Returns total size of category database - total count of points
function CatSize: Integer;
begin
 Result := cat_db.GetTableValue('SELECT count(*) FROM cat');
end;

//Returns count of points in given category
function CatCategorySize(category : string) : Integer;
begin
   cat_db.AddParamText(':cat',category);
 Result := cat_db.GetTableValue('SELECT count(*) FROM cat WHERE category LIKE :cat'); 
end;

//Frees up the category database. 
procedure catFinish;
begin
  cat_db.free;
end;
